#!/usr/bin/env python

# Networkx 1.7

import sys
import time
import networkx as nx

def fileLen(fname):
    with open(fname) as f:
        for i, l in enumerate(f):
            pass
    return i + 1

def init(buffer):
   from glob import globs, setGlobals
   from array import array

   # create spatial variables of specified number
   temp = buffer.pop(0)
   Vars = int(temp.split()[0].strip())+1
   TypeId = temp.split()[1].strip()
   
   # initialize constraints
   N = fileLen(globs['calc']+"/"+globs['calc']+".relations")
   setGlobals("size", N)
   Id = 0
   from bitcoding import B_dict
   with open(globs['calc']+"/"+globs['calc']+".identity") as f:
      Id = B_dict[f.readline().strip()]
  
   ConMatrix = [None for v in xrange(Vars)]
   for v in xrange(Vars):
      j = {}   
      j[v] = Id
      ConMatrix[v] = j

   # parse spatial CSP
   from parsecsp import parsecsp
   parsecsp(ConMatrix, buffer)
   
   return TypeId, ConMatrix

class Usage(Exception):
    def __init__(self, msg):
        self.msg = msg

# main function
def main(argv=None):

   if argv is None:
        argv = sys.argv

   from optparse import OptionParser

   # initialize parser for command line arguments
   parser = OptionParser()
   
   # set parsing options
   parser.add_option("-c", "--calculus", type="string", dest="calc", default="rcc8", help="interaction mode: rcc8, or allen [default: %default]")
   parser.add_option("-f", "--file", action="store", type="string", dest="filename")

   # parse command line arguments
   options, args = parser.parse_args()

   from glob import setGlobals
   #import argparse

   try:   
      # check and set globals for calculus information
      if not (options.calc == "rcc8" or options.calc == "allen"):
         raise Usage("Invalid option: " + options.calc)
      else:
         setGlobals('calc',options.calc)

   except Usage, err:
        print err.msg
        parser.print_help()
        return 1

   with open(options.filename) as f:
      buffer = []

      for i in f:
         buffer.append(i)

         if i.strip() == '.':

            TypeId, ConMatrix = init(buffer)
            buffer = []

            from bitcoding import B_dict
            DALL = B_dict['DALL']

            edges = set([])

            for n, i in enumerate(ConMatrix):
               for j in i:
                  if j > n and ConMatrix[n][j] != DALL:
                     edges.add((n,j))

            G=nx.Graph()
            G.add_nodes_from(xrange(len(ConMatrix)))
            G.add_edges_from(edges)

            edges = None
            
            graphs = [i for i in nx.biconnected_component_subgraphs(G) if i.number_of_nodes() > 2]
            
            from basesplit import bsplit
            from helpfuncs import translate

            meter = 0
            while graphs:
               g = graphs.pop()

               print str(g.number_of_nodes()) + " " + str(TypeId) + "_" + str(meter)

               mynodes = tuple(g.nodes())
               newnodes = {}
               counter = 0
               for n in mynodes:
                  newnodes[n] = counter
                  counter += 1
               
               for i,j in g.edges():
                  if i < j:
                     print str(newnodes[i]) + " " + str(newnodes[j]) + " " + "( " + ' '.join([translate(p) for p in bsplit[ConMatrix[i][j]-1][1]]) + " )"
                  else:
                     print str(newnodes[j]) + " " + str(newnodes[i]) + " " + "( " + ' '.join([translate(p) for p in bsplit[ConMatrix[j][i]-1][1]]) + " )"

               print "."
               meter += 1
            

if __name__ == '__main__':
   sys.exit(main())
